@@ -3,5 +3,3 @@
 from .meshgrid import *
 from .grid_dim import *
 from .indexer import *
-from .xy_dim import *
-# from .datetime_dim import *
