@@ -2,6 +2,8 @@
 
 import six
 import numpy as np
+from functools import reduce
+from operator import add
 
 __all__ = ['XYTiler']
 
@@ -15,7 +17,23 @@ class XYTiler(object):
         "upperright": (1.0, 1.0)
     }
 
-    def __init__(self, x_size, y_size, nx, ny, x0=0.0, y0=0.0, margin=0, i_min=None, i_max=None, j_min=None, j_max=None, **kwargs):
+    def __init__(self, x_size, y_size, nx, ny, x0=0.0, y0=0.0, margin=0, i_min=None, i_max=None, j_min=None, j_max=None, circular_xs=None, **kwargs):
+        """
+
+        :param x_size:
+        :param y_size:
+        :param nx:
+        :param ny:
+        :param x0:
+        :param y0:
+        :param margin:
+        :param i_min:
+        :param i_max:
+        :param j_min:
+        :param j_max:
+        :param circular_xs: None or [x1, x2] (e.g. [-180.0, 180.0])
+        :param kwargs:
+        """
         self.x_size = x_size
         self.y_size = y_size
 
@@ -35,6 +53,8 @@ class XYTiler(object):
         self.j_min = j_min
         self.j_max = j_max
 
+        self.circular_xs = circular_xs
+
     @property
     def full_nx(self):
         return self.nx + self.margin * 2
@@ -43,7 +63,9 @@ class XYTiler(object):
     def full_ny(self):
         return self.ny + self.margin * 2
 
-    def xy2tile(self, x, y, margin=True):
+    def xy2tile(self, x, y, margin=True, ignore_circular=False):
+        if not ignore_circular:
+            x = self.circular_shift(x)
         tile_i, i = self._to_tile_1d(x, self.x0, self.x_size, self.nx)
         tile_j, j = self._to_tile_1d(y, self.y0, self.y_size, self.ny)
         if margin:
@@ -79,10 +101,106 @@ class XYTiler(object):
         y2 = y1 + self.y_size
         return (x1, y1, x2, y2)
 
-    def get_covered_tiles(self, x1, y1, x2, y2, padding=0, detail=False, margin=True):
+    def circular_shift(self, x):
+        if not self.circular_xs:
+            return x
+        cx1, cx2 = self.circular_xs
+
+        if x < cx1:
+            return x + (cx2 - cx1)
+        elif x >= cx2:
+            return x - (cx2 - cx1)
+        else:
+            return x
+
+    def circular_shift_xs(self, x1, x2, direction=None):
+        if not self.circular_xs:
+            return x1, x2
+        cx1, cx2 = self.circular_xs
+        span = cx2 - cx1
+
+        if direction == 'right':
+            return x1 + span, x2 + span
+        elif direction == 'left':
+            return x1 - span, x2 - span
+        else:
+            return x1, x2
+
+    def get_covered_tiles(self, x1, y1, x2, y2, padding=0, detail=False, margin=True, circular_handled=False):
+        if self.circular_xs and not circular_handled:
+            cx1, cx2 = self.circular_xs
+
+            # split range [x1, x2] into several parts
+            parts = []
+            rest_x1 = x1
+            while rest_x1 is not None:
+                if rest_x1 < cx1:
+                    if x2 <= cx1:
+                        parts.append(self.circular_shift_xs(rest_x1, x2, direction='right'))
+                        break
+                    else:
+                        parts.append(self.circular_shift_xs(rest_x1, cx1, direction='right'))
+                        rest_x1 = cx1
+                elif rest_x1 < cx2:
+                    if x2 <= cx2:
+                        parts.append((rest_x1, x2))
+                        break
+                    else:
+                        parts.append((rest_x1, cx2))
+                        rest_x1 = cx2
+                else:
+                    parts.append(self.circular_shift_xs(rest_x1, x2, direction='left'))
+                    break
+
+            part_results = [self.get_covered_tiles(px1, y1, px2, y2, padding=padding, detail=detail, margin=margin, circular_handled=True) for (px1, px2) in parts]
+
+            if detail:
+                final_result = {
+                    'nj': part_results[0]['nj']
+                }
+
+                for key in ['ni', 'tile_list']:
+                    value_list = [pr[key] for pr in part_results]
+                    final_result[key] = reduce(add, value_list)
+
+                def merge_dict(a, b):
+                    a.update(b)
+                    return a
+
+                for key in ['i_beg_dict', 'i_end_dict', 'j_beg_dict', 'j_end_dict', 'j_offset_dict']:
+                    value_list = [pr[key] for pr in part_results]
+                    final_result[key] = reduce(merge_dict, value_list, {})
+
+                offoff = 0
+                final_i_offset_dict = {}
+                for pr in part_results:
+                    for k, v in pr['i_offset_dict'].items():
+                        final_i_offset_dict[k] = v + offoff
+                    offoff += pr['ni']
+
+                final_result['i_offset_dict'] = final_i_offset_dict
+
+                # when generating total_xs and total_ys, circular is not considered.
+                tile_i1, tile_j1, i1, j1 = self.xy2tile(x1, y1, margin=False, ignore_circular=True)
+                tile_i2, tile_j2, i2, j2 = self.xy2tile(x2, y2, margin=False, ignore_circular=True)
+                x_beg, y_beg = self.tile2xy(tile_i1, tile_j1, i1, j1, margin=False)
+                x_end, y_end = self.tile2xy(tile_i2, tile_j2, i2, j2, margin=False)
+                total_xs = np.linspace(x_beg, x_end, final_result['ni'])
+                total_ys = np.linspace(y_beg, y_end, final_result['nj'])
+                final_result.update({
+                    "xs": total_xs,
+                    "ys": total_ys,
+                })
+            else:
+                final_result = []
+                for part_result in part_results:
+                    final_result.extend(part_result)
+
+            return final_result
+
         # margin is considered only when generating each tile's details.
-        tile_i1, tile_j1, i1, j1 = self.xy2tile(x1, y1, margin=False)
-        tile_i2, tile_j2, i2, j2 = self.xy2tile(x2, y2, margin=False)
+        tile_i1, tile_j1, i1, j1 = self.xy2tile(x1, y1, margin=False, ignore_circular=True)
+        tile_i2, tile_j2, i2, j2 = self.xy2tile(x2, y2, margin=False, ignore_circular=True)
 
         x2_, y2_ = self.tile2xy(tile_i2, tile_j2, i2, j2, pos='lowerleft', margin=False)
         if i2 == 0:
